@@ -5,7 +5,7 @@ import warnings
 
 import numpy as np
 
-from numba import typed, njit, errors
+from numba import typed, njit, errors, typeof
 from numba.core import types
 from numba.experimental import structref
 from numba.extending import overload_method, overload_attribute
@@ -58,6 +58,9 @@ class MyStruct(structref.StructRefProxy):
     def prop(self):
         return self.values, self.counter
 
+    def __hash__(self):
+        return compute_fields(self)
+
 
 @structref.register
 class MyStructType(types.StructRef):
@@ -123,6 +126,20 @@ def compute_fields(st):
     return st.values + st.counter
 
 
+@njit
+def test_structref_is():
+    c = MyStruct(3, 4)
+    d = MyStruct(3, 4)
+    return (c is c, c is d)
+
+
+class TestStructRefIs(TestCase):
+    def test_is_identity(self):
+        res = test_structref_is()
+        self.assertTrue(res[0])
+        self.assertFalse(res[1])
+
+
 class TestStructRefBasic(MemoryLeakMixin, TestCase):
     def test_structref_type(self):
         sr = types.StructRef([('a', types.int64)])
@@ -226,6 +243,14 @@ class TestStructRefBasic(MemoryLeakMixin, TestCase):
             # the second field is now an integer.
             td['b'] = MyStruct(2.3, 1)
 
+    def test_MyStructType_hash_no_typeof_recursion(self):
+        # Tests that __hash__ is not called prematurely in typeof
+        # causing infinite recursion (see #8241).
+        st = MyStruct(1, 2)
+        typeof(st)
+
+        self.assertEqual(hash(st), 3)
+
 
 @overload_method(MyStructType, "testme")
 def _ol_mystructtype_testme(self, arg):
